#include "gomoku.h"

int GoMoku::dx[4] = { 0,-1,-1,-1};
int GoMoku::dy[4] = {-1,-1, 0, 1};


void GoMoku::Move::write(ostream &o) const{
  o << x << " " << y << endl; 
}

void GoMoku::Move::read(istream &i){
  i >> x >> y;
}

GoMoku::GoMoku() : stones(0){
  for (int i=0 ; i<BOARD_SIZE ; i++)
    for (int j=0 ; j<BOARD_SIZE ; j++)
      board[i][j] = 0;
}


void GoMoku::dump(ostream &o) const{
  char tr[3] = {'.','X','O'};
  for (int i=0 ; i<BOARD_SIZE ; i++){
    for (int j=0 ; j<BOARD_SIZE ; j++)
      o << tr[board[i][j]];
    o << endl;
  }
}


vector< ::Move *> GoMoku::getLegalMoves() const{
  vector< ::Move *> moves;
  for (int i=0 ; i<BOARD_SIZE ; i++)
    for (int j=0 ; j<BOARD_SIZE ; j++)
      if (board[i][j]==0)
	moves.push_back(new GoMoku::Move(i,j));
  return moves;
}


int GoMoku::commitMove(const ::Move *_move){
  Move *move = (Move*) _move;
  
  int p =  getPlayerToMove(); 
  int x = move->getX(); 
  int y = move->getY();
  
  ply++;
  stones++;
  
  board[x][y] = p;
  
  for(int d = 0; d < 4; d++){
    int forward = 1;
    while ( isLegal(x+forward*dx[d],y+forward*dy[d]) && board[x+forward*dx[d]][y+forward*dy[d]] == p)
      forward++;
    int backward = 1;
    while ( isLegal(x-backward*dx[d],y-backward*dy[d]) && board[x-backward*dx[d]][y-backward*dy[d]] == p)
      backward++;
    if (forward+backward > LINE_SIZE){
      state = p;
      return p;
    }
  }
  
  if (stones == BOARD_SIZE*BOARD_SIZE){
    state = TIE;
    return state;
  }
  return 0;
};


void GoMoku::undoMove(const ::Move *_move){
  Move *move = (Move*) _move;
  board[move->getX()][move->getY()] = 0;
  ply--;
  stones--;
  state=0;
};


int GoMoku::evaluate() const{
  int p = getPlayerToMove();
  int eval = 0;
  for (int x=0 ; x<BOARD_SIZE ; x++)
    for (int y=0; y<BOARD_SIZE ; y++)  
      if (board[x][y]==p){
	for(int d = 0; d < 4; d++){
	  int size = 1;
	  while (isLegal(x+size*dx[d],y+size*dy[d]) && (board[x+size*dx[d]][y+size*dy[d]] == p))
	    size++;
	  eval += size - 1;
	}
      }
  return eval;
}
